"""
nginx 配置生成工具
"""
from dataclasses import asdict, dataclass
from datetime import datetime
import logging
from pathlib import Path
import shutil
import subprocess

from logger import get_logger
logger = get_logger(__name__)

NGINX_CONF_DIR = Path("nginx/conf/conf.d")
BACKUP_DIR = Path("nginx/conf/backups")
CONTAINER_NAME = "nginx"

@dataclass
class NginxConfig:
    """
    表示一个 Nginx 配置项的数据类。

    属性:
        name (str): 配置项名称，默认为 "unknown"。
        host (str): 主机地址，默认为 "localhost"。
        port (int): 端口号，默认为 80。
        output_path (Optional[str]): 输出路径，默认为 None。
        tcp_nopush (str): TCP 推送选项，默认为 "off"。
        sendfile (str): Sendfile 选项，默认为 "off"。
        client_max_body_size (str): 客户端最大请求体大小，默认为 "100m"。
        proxy_connect_timeout (str): 代理连接超时时间，默认为 "60s"。
        proxy_send_timeout (str): 代理发送超时时间，默认为 "60s"。
        proxy_read_timeout (str): 代理读取超时时间，默认为 "60s"。
        web_socket_proxy (bool): 是否启用 WebSocket 代理，默认为 False。
    """
    name: str = "unknown"
    host: str = "localhost"
    port: int = 80
    output_path: str | None = None
    tcp_nopush: str = "off"
    sendfile: str = "off"
    client_max_body_size: str = "100m"
    proxy_connect_timeout: str = "60s"
    proxy_send_timeout: str = "60s"
    proxy_read_timeout: str = "60s"
    web_socket_proxy: bool = False

@dataclass
class NginxConfigurator:
    """
    Nginx配置管理类，封装配置生成、备份、重载等操作
    
    属性:
        compose_file (Path): Docker Compose文件路径
        output_dir (Path): 配置文件输出目录
        backup_dir (Path): 配置备份目录
        logger (logging.Logger): 日志记录器
    """

    output_dir: Path = NGINX_CONF_DIR
    backup_dir: Path = BACKUP_DIR
    logger: logging.Logger = logger

    def gen_all_config(self, configs: list[NginxConfig]) -> str:
        """生成Nginx配置内容"""
        return "\n".join([self.gen_once_config(conf) for conf in configs])

    def gen_once_config(self, conf: NginxConfig) -> str:
        """构建单个server配置块"""
        template = \
"""
server {{
    # # 必须添加DNS解析器配置
    # resolver 127.0.0.11 valid=10s;  # Docker内置DNS服务
    
    # upstream {name}_upstream {{
    #     zone upstream_{name} 64k;  # 共享内存区
    #     server {host}:{port} resolve max_fails=3 fail_timeout=30s;
    #     server 0.0.0.0:65535 backup;  # 占位无效地址
    # }}

    listen 80;
    listen 443 ssl;
    server_name {name};
    if ($scheme = http) {{
        return 301 https://$server_name$request_uri;
    }}

    tcp_nopush {tcp_nopush};
    sendfile {sendfile};
    client_max_body_size {client_max_body_size};
    location / {{
        proxy_pass http://{host}:{port};
        proxy_connect_timeout {proxy_connect_timeout};
        proxy_send_timeout {proxy_send_timeout};
        proxy_read_timeout {proxy_read_timeout};
        proxy_set_header Host $host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-Forwarded-Proto $scheme;
        {_web_socket_proxy}
    }}
}}
"""
        conf_dict = {
            "_web_socket_proxy": 
        """
        proxy_set_header Upgrade $http_upgrade;
        proxy_set_header Connection "upgrade";
        """.strip() if conf.web_socket_proxy else "",
        }
        return template.format_map(asdict(conf) | conf_dict)

    def backup_config(self, filename: Path = Path("generated.conf")) -> Path | None:
        """执行配置文件备份"""
        try:
            self.backup_dir.mkdir(exist_ok=True, parents=True)
            backup_file = self.backup_dir / f"{filename}.bak.{datetime.now():%Y%m%d-%H%M%S}"

            if not (self.output_dir / filename).exists():
                self.logger.warning("未找到源文件，无法备份")
                return None
            shutil.copy(self.output_dir / filename, backup_file)
            self.logger.info("配置已备份至 %s", backup_file)
            return backup_file
        except Exception as e:
            self.logger.error("备份失败: %s", str(e))
            return None

    def gen_and_save_config(self, configs: list[NginxConfig],
                   filename: Path = Path("generated.conf")) -> bool:
        """生成配置文件并保存到文件里"""
        try:
            output_file = self.output_dir / filename
            output_file.parent.mkdir(exist_ok=True, parents=True)

            with open(output_file, 'w', encoding='utf-8') as f:
                res = self.gen_all_config(configs)
                f.write(res)
            logger.info("配置已写入 %s", output_file)
            return True
        except Exception as e:
            self.logger.error(f"生成文件失败: {str(e)}")
            return False

    def safe_reload(self) -> bool:
        """安全重载Nginx配置"""
        if self.test_config() and self.reload():
            return True
        return False

    def test_config(self) -> bool:
        """测试Nginx配置"""
        result = subprocess.run(
            ["docker", "exec", CONTAINER_NAME, "nginx", "-t"],
            capture_output=True, stderr=None, stdout=None, check=False
        )
        if result.returncode != 0:
            self.logger.error("配置测试失败: %s", result.stderr)
            return False
        else:
            return True

    def reload(self) -> bool:
        """执行配置重载"""
        try:
            result = subprocess.run(
                ["docker", "exec", CONTAINER_NAME, "nginx", "-s", "reload"],
                capture_output=True, stderr=None, stdout=None, check=False
            )
            if result.returncode != 0:
                self.logger.error("配置重载失败: %s", result.stderr)
                return False
            else:
                self.logger.info("配置重载成功")
                return True
        except subprocess.CalledProcessError as e:
            self.logger.error("重载失败: %s", str(e))
            return False

    def rollback(self) -> bool:
        """回滚到最近的有效配置"""
        raise NotImplementedError()
        # backups = sorted(self.backup_dir.glob("*.bak.*"),
        #                key=os.path.getmtime, reverse=True)
        # if not backups:
        #     self.logger.error("无可用的备份配置")
        #     return False
        # try:
        #     shutil.copy(backups[0], self.output_dir / "generated.conf")
        #     self.logger.warning(f"已回滚到备份 {backups[0].name}")
        #     return self.safe_reload()
        # except Exception as e:
        #     self.logger.error(f"回滚失败: {str(e)}")
        #     return False

if __name__ == '__main__':
    config = NginxConfigurator()
    config.safe_reload()
    # conf = NginxConfig()
    # conf.web_socket_proxy = True
    # ret = config.gen_config([conf, conf])
    # config.backup_config()
